
# ==========================================
# This program derives different analytical 
# distributions and, if requested, std dev associated with them
# ==========================================

# HHH is a vector of considered horizons, expressed in months:
HHH <- seq(12,120,by=12)

Select.if.compute.stdv <- matrix(0,length(HHH),estimated.Model$r)

# Compute smoothed results:
KF.res.0 <- KF.res

# Get smoothed factors:
X <- KF.res.0$r_smooth


step.distri <- .25/2
step.4.integral <- .05
center.distri <- 2
width.distri <- 7


step.4.integral <- .02
max.v.4.integral <- 50

# Number of considered abscissa:
nb.abscissa <- width.distri/step.distri


# PDF (and CDF) results will be in array of dimension:
#      T x (nb points where pdf evaluated) x (nb horizons) X (nb countries)
PDF.all <- array(NaN,c(dim(X)[1],nb.abscissa-1,length(HHH),estimated.Model$r))
CDF.all <- array(NaN,c(dim(X)[1],nb.abscissa-1,length(HHH),estimated.Model$r))

PDF.stdv.all <- array(NaN,c(dim(X)[1],nb.abscissa-1,length(HHH),estimated.Model$r))
CDF.stdv.all <- array(NaN,c(dim(X)[1],nb.abscissa-1,length(HHH),estimated.Model$r))

PDF.average.all <- array(NaN,c(dim(X)[1],nb.abscissa-1,length(HHH),estimated.Model$r))
CDF.average.all <- array(NaN,c(dim(X)[1],nb.abscissa-1,length(HHH),estimated.Model$r))

PDF.average.stdv.all <- array(NaN,c(dim(X)[1],nb.abscissa-1,length(HHH),estimated.Model$r))
CDF.average.stdv.all <- array(NaN,c(dim(X)[1],nb.abscissa-1,length(HHH),estimated.Model$r))

PDF.US.SPF.all <- array(NaN,c(dim(X)[1],nb.abscissa-1,length(HHH),estimated.Model$r))
CDF.US.SPF.all <- array(NaN,c(dim(X)[1],nb.abscissa-1,length(HHH),estimated.Model$r))

PDF.US.SPF.stdv.all <- array(NaN,c(dim(X)[1],nb.abscissa-1,length(HHH),estimated.Model$r))
CDF.US.SPF.stdv.all <- array(NaN,c(dim(X)[1],nb.abscissa-1,length(HHH),estimated.Model$r))

PDF.x.all <- NULL

for(indic.average in 0:1){
  print(paste("Type of inflation: ",toString(indic.average+1)," (out of 2)",sep=""))
  for(area in 1:estimated.Model$r){
    print(paste("    Area: ",toString(area)," (out of 2)",sep=""))
    b <- c(estimated.Model$delta[,area],rep(0,estimated.Model$q))
    max.x <- log(max.v.4.integral)
    v <- matrix(exp(seq(-10,max.x,by=step.4.integral)),nrow=1)
    
    dim.v <- length(v)
    u_Y <- 1i * matrix(b[1:estimated.Model$n],ncol=1) %*% v 
    u_z <- 1i * matrix(b[(estimated.Model$n+1):(estimated.Model$n+estimated.Model$q)],ncol=1) %*% v  # our paper, section 7.3 Appendix i*v*gamma for z
    AB.list <- compute_AB(estimated.Model,
                          u_Y,u_z,
                          HHH,
                          indic.average)
    
    min.bx <- center.distri - estimated.Model$pi.bar[area] - width.distri/2
    max.bx <- center.distri - estimated.Model$pi.bar[area] + width.distri/2
    
    count <- 0
    for(HH in HHH){
      count <- count + 1
      if(Select.if.compute.stdv[count,area]==1){
        Sigma.X <- KF.res.0$S_smooth
      }else{
        Sigma.X <- 0
      }
      
      indic.matur <- which(HH==HHH)
      AB.list.external <- list(A=array(AB.list$A[,,indic.matur],c(dim(AB.list$A)[1],dim(AB.list$A)[2],1)),
                               B=array(AB.list$B[,,indic.matur],c(dim(AB.list$B)[1],dim(AB.list$B)[2],1)))
      
      system.time(
        res.analytical.computation <-
          compute.distri.plus.stdv(estimated.Model,X,b,HH,
                                   min.bx,max.bx,step.distri,
                                   step.4.integral,
                                   max.v.4.integral,
                                   indic.average,
                                   Sigma.X,
                                   AB.list.external)
      )
      
      if(indic.average==0){
        PDF.all[,,count,area] <- res.analytical.computation$PDF
        CDF.all[,,count,area] <- res.analytical.computation$CDF
        if(Select.if.compute.stdv[count,area]==1){# want to compute stdev of distributions
          PDF.stdv.all[,,count,area] <- res.analytical.computation$PDF.stdv
          CDF.stdv.all[,,count,area] <- res.analytical.computation$CDF.stdv
        }
      }
      if(indic.average==1){
        PDF.average.all[,,count,area] <- res.analytical.computation$PDF
        CDF.average.all[,,count,area] <- res.analytical.computation$CDF
        if(Select.if.compute.stdv[count,area]==1){# want to compute stdev of distributions
          PDF.average.stdv.all[,,count,area] <- res.analytical.computation$PDF.stdv
          CDF.average.stdv.all[,,count,area] <- res.analytical.computation$CDF.stdv
        }
      }
      if(indic.average==2){
        PDF.US.SPF.all[,,count,area] <- res.analytical.computation$PDF
        CDF.US.SPF.all[,,count,area] <- res.analytical.computation$CDF
        if(Select.if.compute.stdv[count,area]==1){# want to compute stdev of distributions
          PDF.US.SPF.stdv.all[,,count,area] <- res.analytical.computation$PDF.stdv
          CDF.US.SPF.stdv.all[,,count,area] <- res.analytical.computation$CDF.stdv
        }
      }
    }
    PDF.x.all <- rbind(PDF.x.all,res.analytical.computation$PDF.x)
  }
}


FILE = paste("PDFs/analytical.pdfs.",estimated.Model$areas,
             ".n",toString(estimated.Model$n),
             ".q",toString(estimated.Model$q),
             ".res",sep="")

save(PDF.all,PDF.stdv.all,CDF.all,CDF.stdv.all,
     PDF.average.all,PDF.average.stdv.all,CDF.average.all,CDF.average.stdv.all,
     PDF.US.SPF.all,PDF.US.SPF.stdv.all,CDF.US.SPF.all,CDF.US.SPF.stdv.all,
     PDF.x.all,
     HHH,Select.if.compute.stdv,indic.average,
     file=FILE)



