
# ==========================================
# This program produces plots showing the 
# decompositon of anchoring measure into
# no-stochastic vol versus stochastic vol parts.
# ==========================================


vec.lower.bounds <- c(1.5,1.5)
vec.upper.bounds <- c(2.5,2.5)

vec.HH <- c(60,120)

vec.indic.average <- c(0,1) # in 0,1,2 # 1 for averages over periods

# Compute smoothed results:
KF.res.0 <- KF.res

# Get smoothed factors:
X <- KF.res.0$r_smooth
# Version of factors where the vol factors are set at their uncond means:
X.no.stoch.vol <- X
X.no.stoch.vol[,(n+1):(n+q)] <- matrix(1,dim(X)[1],1) %*%
  matrix(estimated.Model$mu.z / (1 - diag(estimated.Model$Phi.z)),nrow=1)

Sigma.X <- KF.res.0$S_smooth

# ======================================
# Calibration of parameters 
step.4.integral <- .02
max.v.4.integral <- 50
# ======================================

max.x <- log(max.v.4.integral)
v <- matrix(exp(seq(-10,max.x,by=step.4.integral)),nrow=1)
dim.v <- length(v)


HHH <- seq(12,120,by=12)


all.proba.EA <- NULL
all.proba.no.stoch.vol.EA <- NULL
all.lower.bounds.EA <- NULL
all.upper.bounds.EA <- NULL
all.proba.US <- NULL
all.proba.no.stoch.vol.US <- NULL
all.lower.bounds.US <- NULL
all.upper.bounds.US <- NULL
all.proba.diff <- NULL
all.lower.bounds.diff <- NULL
all.upper.bounds.diff <- NULL


for(case in 1:length(vec.HH)){
  
  indic.average <- vec.indic.average[case]
  
  
  for(area in 1:2){
    b <- c(estimated.Model$delta[,area],rep(0,estimated.Model$q))
    u_Y <- 1i * matrix(b[1:estimated.Model$n],ncol=1) %*% v 
    u_z <- 1i * matrix(b[(estimated.Model$n+1):(estimated.Model$n+estimated.Model$q)],ncol=1) %*% v  # our paper, section 7.3 Appendix i*v*gamma for z
    AB.list <- compute_AB(estimated.Model,u_Y,u_z,HHH,indic.average)
    
    if(area==1){
      AB.list.1 <- AB.list
    }else{
      AB.list.2 <- AB.list
    }
  }
  
  HH <- vec.HH[case]
  
  lower.bound <- vec.lower.bounds[case]
  upper.bound <- vec.upper.bounds[case]
  
  indic.matur <- which(HH==HHH)
  
  
  for(area in 1:2){
    
    print(paste("Case: ",toString(case),"; Area: ",toString(area),sep=""))
    
    b <- c(estimated.Model$delta[,area],rep(0,estimated.Model$q))
    
    lower.bound.bX <- lower.bound - estimated.Model$pi.bar[area]
    upper.bound.bX <- upper.bound - estimated.Model$pi.bar[area]
    
    if(area==1){
      AB.list <- AB.list.1
    }else{
      AB.list <- AB.list.2
    }
    AB.list.external <- list(A=array(AB.list$A[,,indic.matur],c(dim(AB.list$A)[1],dim(AB.list$A)[2],1)),
                             B=array(AB.list$B[,,indic.matur],c(dim(AB.list$B)[1],dim(AB.list$B)[2],1)))
    
    print(indic.average)
    
    res.analytical.computation <-
      compute.proba.interval.plus.stdv(estimated.Model,
                                       X,
                                       b,HH,lower.bound.bX,upper.bound.bX,
                                       step.4.integral,max.v.4.integral,indic.average,
                                       Sigma.X,AB.list.external)
    
    res.analytical.computation.no.stoch.vol <-
      compute.proba.interval.plus.stdv(estimated.Model,
                                       X.no.stoch.vol,
                                       b,HH,lower.bound.bX,upper.bound.bX,
                                       step.4.integral,max.v.4.integral,indic.average,
                                       Sigma.X,AB.list.external)
    
    plot(res.analytical.computation$Proba,type="l")
    lines(res.analytical.computation$Proba - 2*res.analytical.computation$Proba.stdv,lty=2)
    lines(res.analytical.computation$Proba + 2*res.analytical.computation$Proba.stdv,lty=2)
    
    if(area==1){
      all.proba.EA <- cbind(all.proba.EA,res.analytical.computation$Proba)
      all.proba.no.stoch.vol.EA <- cbind(all.proba.no.stoch.vol.EA,
                                         res.analytical.computation.no.stoch.vol$Proba)
      all.lower.bounds.EA <- cbind(all.lower.bounds.EA,res.analytical.computation$Proba - 2*res.analytical.computation$Proba.stdv)
      all.upper.bounds.EA <- cbind(all.upper.bounds.EA,res.analytical.computation$Proba + 2*res.analytical.computation$Proba.stdv)
    }else{
      all.proba.US <- cbind(all.proba.US,res.analytical.computation$Proba)
      all.proba.no.stoch.vol.US <- cbind(all.proba.no.stoch.vol.US,
                                         res.analytical.computation.no.stoch.vol$Proba)
      all.lower.bounds.US <- cbind(all.lower.bounds.US,res.analytical.computation$Proba - 2*res.analytical.computation$Proba.stdv)
      all.upper.bounds.US <- cbind(all.upper.bounds.US,res.analytical.computation$Proba + 2*res.analytical.computation$Proba.stdv)
    }
    
  }
  
  # Differences of proba:
  AB.list <- AB.list.1
  AB.list.external.1 <- list(A=array(AB.list$A[,,indic.matur],c(dim(AB.list$A)[1],dim(AB.list$A)[2],1)),
                             B=array(AB.list$B[,,indic.matur],c(dim(AB.list$B)[1],dim(AB.list$B)[2],1)))
  AB.list <- AB.list.2
  AB.list.external.2 <- list(A=array(AB.list$A[,,indic.matur],c(dim(AB.list$A)[1],dim(AB.list$A)[2],1)),
                             B=array(AB.list$B[,,indic.matur],c(dim(AB.list$B)[1],dim(AB.list$B)[2],1)))
  
  res.diff.proba <- compute.difference.proba.plus.stdv(estimated.Model,X,HH,
                                                       lower.bound,upper.bound,
                                                       step.4.integral,max.v.4.integral,indic.average,
                                                       Sigma.X,AB.list.external.1,AB.list.external.2)
  
  all.proba.diff <- cbind(all.proba.diff,res.diff.proba$Proba.diff)
  all.lower.bounds.diff <- cbind(all.lower.bounds.diff,res.diff.proba$Proba.diff - 2*res.diff.proba$Proba.diff.stdv)
  all.upper.bounds.diff <- cbind(all.upper.bounds.diff,res.diff.proba$Proba.diff + 2*res.diff.proba$Proba.diff.stdv)
  
}


# ===========================================================
# Prepare graphics
# ===========================================================

FILE = "/OUTPUTS/Figure_anchoring_proba_withNoStochVol.pdf"
pdf(file=paste(getwd(),FILE,sep=""),pointsize=10,width=7, height=9)

par(mfrow=c(length(vec.indic.average),1))
par(plt=c(.1,.95,.1,.85))

for(i in 1:length(vec.indic.average)){
  
  if(i==1){
    main.tit <- expression(paste(
      "1-year 4-years-ahead inflation, ",
      pi[t+48*","*t+60],sep=""))}
  if(i==2){
    main.tit <- expression(paste(
      "5-year 5-years-ahead inflation, ",
      pi[t+60*","*t+120],sep=""))}
  
  if(i<2){
    y.lim <- c(0.2,.8)
  }else{
    y.lim <- c(0.2,.8)
  }
  
  plot(vec.dates,rep(0,length(vec.dates)),type="l",
       ylim=y.lim,
       xlab="",ylab="",
       main=main.tit,col="white")
  
  for(area in 1:2){
    if(area==1){
      Area <- "Euro area"
      rec.dates <- EA.rec.dates
      col.recess.areas <- "#EECCEEBB"
    }
    if(area==2){
      Area <- "United States"
      rec.dates <- US.rec.dates
      col.recess.areas <- "#CCEEEEBB"
    }
    
    #------ Add recession areas ------
    nb.rec <- length(rec.dates)/8
    polygon(rec.dates,c(rep(c(-10000,10000,10000,-10000),nb.rec),rep(-10000,nb.rec*4)),
            col=col.recess.areas,border=NA)
    #---------------------------------
  }
  
  
  announc.infl.target <- as.Date("15-05-2003","%d-%m-%Y")
  lines(c(announc.infl.target,announc.infl.target),c(-100,100),lwd=2,lty=2,col="red")
  announc.infl.target <- as.Date("15-01-2012","%d-%m-%Y")
  lines(c(announc.infl.target,announc.infl.target),c(-100,100),lwd=2,col="red")
  
  
  for(area in 1:2){
    if(area==1){
      proba <- all.proba.EA[,i]
      proba.no.stoch.vol <- all.proba.no.stoch.vol.EA[,i]
      lower.bound <- all.lower.bounds.EA[,i]
      upper.bound <- all.upper.bounds.EA[,i]
      color.l <- "dark gray"
    }else{
      proba <- all.proba.US[,i]
      proba.no.stoch.vol <- all.proba.no.stoch.vol.US[,i]
      lower.bound <- all.lower.bounds.US[,i]
      upper.bound <- all.upper.bounds.US[,i]
      color.l <- "black"
    }
    
    polygon(c(vec.dates,vec.dates[length(vec.dates):1]),c(lower.bound,upper.bound[length(vec.dates):1]),
            col="#90909044",border=NA)
    lines(vec.dates,proba,col=color.l,
          lwd=2)
    lines(vec.dates,proba.no.stoch.vol,col=color.l,
          lwd=2,lty=2)
  }
  
  diff.proba <- all.proba.diff[,i]
  diff.lower <- all.lower.bounds.diff[,i]
  diff.upper <- all.upper.bounds.diff[,i]
  
  abline(h=0,col="dark grey")
  
  
  if(i==1){
    legend("bottomright",
           c("Euro Area (Part A + Part B)","Euro Area (Part A)","United States (Part A + Part B)","United States (Part A)"),
           lty=c(1,2,1,2),
           lwd=c(2,2,2,2),
           col=c("dark grey","dark grey","black","black"), 
           cex=1,bg="white",seg.len=4)  
  }
  
}

dev.off()



