# *****************************************************************************#
# Measuring Inflation Anchoring and Uncertainty: A US and Euro Area Comparison  
#
# Olesya Grishchenko, Sarah Mouabbi and Jean-Paul Renne.
#
# January 2019
#
# *****************************************************************************#


# Clear environment and console:
rm(list = ls(all = TRUE)) # clear environment 
# clear console:
cat("\014")


# ===================================================
# ===================================================
# Set working directory:
setwd("~/Dropbox/inf_uncertainty/Codes_GMR_JMCB")
# ===================================================
# ===================================================



library(bitops)
library(RCurl)
library(optimx)
library(sandwich)
library(gdata)
library(MASS)
library(compiler)
library(stringr)


par(plt=c(.1,.9,.1,.9))

# Load procedures:
source("./procedures.R")

# Specify model:
n <- 5 # number of Y factors
q <- 2 # number of z factors

# Indicate whether PDFs are computed:
#   (if 1, then "compute.analytical.distri.R" is run; can take a few minutes of computation)
indic.whether.recompute.PDFs <- 1

# Load data:
vec.dates <- read.csv("DATA/DATA_dates.csv", sep=";")
vec.dates <- as.Date(vec.dates[,2],"%m.%d.%Y")

# Load factors:
DATA_factors <- read.csv2("DATA/DATA_factors.csv")
DATA_factors <- DATA_factors[,2:(1+n+q)]

# Load model:
load(file="MODEL/estimated.model.Rdat")


# PREPARE FIGURES --------------------------
source("./make.factor.loadings.figure.R")
source("./make.term.structures.figures.R")
source('./make.times.series.moments.figures.R')
source('./make.anchoring.probas.figures.R')
source('./compute.decomposition.anchoring.R')



