# Prepare factor loadings figure:

# Load Model:
Model <- estimated.Model

FILE = "/OUTPUTS/Figure_loadings.pdf"
pdf(file=paste(getwd(),FILE,sep=""),pointsize=10,width=7, height=9)


unc.mom <- unc.moments.X(estimated.Model)

avg.condi.var <- estimated.Model$Gamma.Y0 +
  t(estimated.Model$Gamma.Y1) %*% unc.mom$X.mom.1[(estimated.Model$n+1):(estimated.Model$n+estimated.Model$q)]

avg.condi.stdv <- sqrt(avg.condi.var)

Gamma <- rbind(estimated.Model$delta,matrix(0,Model$q,Model$r))
S.n <- make.S.p(estimated.Model$n)
S.q <- make.S.p(estimated.Model$q)
H.aux <- seq(1,120,by=1)

par(plt=c(.1,.9,.2,.8))
par(mfrow=c(3,estimated.Model$r))
par(plt=c(.1,.9,.2,.9))

for(area in 1:Model$r){
  if(area==1){Area <- "Euro area"}
  if(area==2){Area <- "United States"}
  loadings <- compute.loadings.c(c(Gamma[,area]),estimated.Model,H.aux,S.n,S.q)
  loadings$b    <- loadings$b[,,2]    
  loadings$beta <- loadings$beta[,,2] 
  
  var.dec <- loadings$b[1:estimated.Model$n,]^2 * (matrix(c(avg.condi.var),ncol=1)%*%matrix(1,1,dim(loadings$b)[2]))
  rel.var.dec <- apply(var.dec,2,cumsum)
  rel.var.dec <- rel.var.dec / (matrix(1,estimated.Model$n,1) %*% matrix(rel.var.dec[estimated.Model$n,],nrow=1))
  
  for(jj in 1:3){
    par(mfg=c(jj,area))
    if(jj<3){
      if(jj==1){
        series.2.plot <- loadings$b[1:estimated.Model$n,]
        main.tit <- paste(Area,", loadings of conditional expectations",sep="")
        symbol <- 21
      }
      if(jj==2){
        series.2.plot <- loadings$beta[(estimated.Model$n+1):(estimated.Model$n+estimated.Model$q),]
        main.tit <- paste(Area,", loadings of conditional variances",sep="")
        symbol <- 22
      }
      y.lim <- c(min(series.2.plot),max(series.2.plot)+.4*(max(series.2.plot)-min(series.2.plot)))
      plot((1:length(H.aux))/12,series.2.plot[1,],ylim=y.lim,
           xlab=expression(paste("Horizon, years (=",h/12,")",sep="")),ylab="",type="l",lty=1,cex=2,
           col="black",bg = rgb(1,1,1), main = main.tit,lwd=3)
      text(.1,y.lim[2]-.04*(y.lim[2]-y.lim[1]),"Loadings of",pos=4)
      if(jj==1){
        text(.1,y.lim[2]-.12*(y.lim[2]-y.lim[1]),expression(paste(E[t],"(",pi[t+h-12*","*t+h],")")),pos=4)
      }else{
        text(.1,y.lim[2]-.12*(y.lim[2]-y.lim[1]),expression(paste(Var[t],"(",pi[t+h-12*","*t+h],")")),pos=4)
      }
      if(dim(series.2.plot)[1]>1){
        lines((1:length(H.aux))/12,series.2.plot[2,],
              lwd=4,lty=3,type="l",cex=2,pch =symbol,
              col="black",bg = rgb(.7,.7,.7))}
      if(dim(series.2.plot)[1]>2){
        lines((1:length(H.aux))/12,series.2.plot[3,],
              lwd=3,lty=1,type="l",cex=2,pch =symbol,
              col="dark grey",bg = rgb(.4,.4,.4))}
      if(dim(series.2.plot)[1]>3){
        lines((1:length(H.aux))/12,series.2.plot[4,],
              lwd=4,lty=3,type="l",cex=2,pch =symbol,
              col="dark grey",bg = rgb(.1,.1,.1))}
      if(dim(series.2.plot)[1]>4){
        lines((1:length(H.aux))/12,series.2.plot[5,],
              lwd=3,lty=2,type="l",cex=2,pch =symbol,
              col="black",bg = rgb(.1,.1,.1))}
      if((area==2)&((jj==1))){
        legend("topright", # places a legend at the appropriate place c("Health","Defense"), # puts text in the legend 
               c(expression(paste("First factor (",Y[1*","*t],")",sep="")),
                 expression(paste("Second factor (",Y[2*","*t],")",sep="")),
                 expression(paste("Third factor (",Y[3*","*t],")",sep="")),
                 expression(paste("Fourth factor (",Y[4*","*t],")",sep="")),
                 expression(paste("Fifth factor (",Y[5*","*t],")",sep=""))
               ),
               lty=c(1,3,1,3,2),
               lwd=c(3,4,3,4,3), # line width
               col=c("black","black","dark grey","dark grey","black"),
               pt.cex = 2,
               cex=1,seg.len=4)  
      }
      if((area==2)&((jj==2))){
        if(estimated.Model$q==3){
          legend("topright",
                 c(expression(paste("First factor (",z[1*","*t],")",sep="")),
                   expression(paste("Second factor (",z[2*","*t],")",sep="")),
                   expression(paste("Third factor (",z[3*","*t],")",sep=""))),
                 lty=c(3,3,3),
                 pch = rep(symbol,4),
                 pt.bg = c(rgb(1,1,1),rgb(0.7,0.7,0.7),rgb(0.4,0.4,0.4)),
                 lwd=c(2,2,2), # line width
                 col=c(rgb(0,0,0),rgb(0.2,0.2,0.2),rgb(0.4,0.4,0.4)), # gives the legend lines the correct color and width
                 #text.width = 2,
                 pt.cex = 2,
                 cex=1,seg.len=4)  
        }else{
          legend("topright",
                 c(expression(paste("First factor (",z[1*","*t],")",sep="")),
                   expression(paste("Second factor (",z[2*","*t],")",sep=""))),
                 lty=c(1,3),
                 lwd=c(3,3),
                 col=c("black","black"),
                 pt.cex = 2,
                 cex=1,seg.len=4)  
        }
      }
      
    }else{# Variance decomposition plot
      main.tit <- paste(Area,", variance decomposition",sep="")
      vec.color <- NULL
      xaxis <- (1:length(H.aux))/12
      nb.xaxis <- length(xaxis)
      rel.var.dec <- rbind(0*rel.var.dec[1,],rel.var.dec)
      plot(xaxis,0*rel.var.dec[1,],type="l",ylim=c(0,1),
           xlab=expression(paste("Horizon, years (=",h/12,")",sep="")),
           ylab="",main=main.tit)
      for(iiiii in 1:estimated.Model$n){
        coloR <- 1 - (iiiii-1)/(estimated.Model$n-1)
        vec.color <- c(vec.color,coloR)
        polygon(c(xaxis,xaxis[nb.xaxis:1]),c(rel.var.dec[iiiii,],rel.var.dec[iiiii+1,nb.xaxis:1]),
                col=rgb(coloR,coloR,coloR),border=NA)
      }
      if(area==2){
        legend("bottomright",
               c(expression(paste("First factor (",Y[1*","*t],")",sep="")),
                 expression(paste("Second factor (",Y[2*","*t],")",sep="")),
                 expression(paste("Third factor (",Y[3*","*t],")",sep="")),
                 expression(paste("Fourth factor (",Y[4*","*t],")",sep="")),
                 expression(paste("Fifth factor (",Y[5*","*t],")",sep=""))
               ),
               lty=c(NaN,NaN,NaN,NaN,NaN),
               pch = rep(22,estimated.Model$n),
               col="black",
               lwd=1, # line width
               pt.bg=c(
                 rgb(vec.color[1],vec.color[1],vec.color[1]),
                 rgb(vec.color[2],vec.color[2],vec.color[2]),
                 rgb(vec.color[3],vec.color[3],vec.color[3]),
                 rgb(vec.color[4],vec.color[4],vec.color[4]),
                 rgb(vec.color[5],vec.color[5],vec.color[5])
               ),
               pt.cex = 2,
               cex=1,seg.len=4,bg="white")  
      }
    }
  }
}


dev.off()


