

Trans.stdev <- function(Model,z){
  # compute standard deviation of transition-eq innovations
  A.prime <- Model$Theta %*% diag(c(sqrt(Model$Gamma.z0 + t(Model$Gamma.z1)%*%z))) 
  B.prime <- diag(c(sqrt(Model$Gamma.Y0 + t(Model$Gamma.Y1)%*%(Model$mu.z + Model$Phi.z %*%z))))
  C.prime <- diag(c(sqrt(Model$Gamma.z0 + t(Model$Gamma.z1)%*%z)))
  D.prime <- matrix(0,Model$q,Model$n)
  Sigma.X1 <- cbind(A.prime,B.prime)
  Sigma.X2 <- cbind(C.prime,D.prime)
  Sigma.X  <- rbind(Sigma.X1,Sigma.X2)
  return(Sigma.X)
}

Qfunction <- function(Model,X,aux){
  z <- X[(Model$n+1):(Model$n+Model$q)]
  # compute covariance matrix of transition-eq innovations
  A <- Model$Theta %*% diag(c(Model$Gamma.z0 + t(Model$Gamma.z1)%*%z)) %*% t(Model$Theta) + 
    diag(c(Model$Gamma.Y0 + t(Model$Gamma.Y1)%*%(Model$mu.z + Model$Phi.z %*%z)))
  B <- diag(c(Model$Gamma.z0 + t(Model$Gamma.z1)%*%z))
  C <- Model$Theta %*% B
  Q1 <- cbind(A,C)
  Q2 <- cbind(t(C),B)
  Q  <- rbind(Q1,Q2)
  return(Q)
}

make.Model <- function(model){
  n <- dim(model$Phi.Y)[1] # number of Y factors
  q <- length(c(model$nu)) # number of z factors
  r <- length(c(model$pi.bar))
  mu.z <- model$mu * model$nu
  Phi.z <- (matrix(model$mu,q,1)%*%matrix(1,1,q)) * t(model$phi)
  Gamma.z0 <- matrix(model$mu,q,1) * matrix(model$mu,q,1) * matrix(model$nu,q,1)
  Gamma.z1 <- 2 * t(((matrix(model$mu,q,1) * matrix(model$mu,q,1))%*%matrix(1,1,q)) *
                      t(model$phi))
  mu.X <- rbind(matrix(- model$Theta %*% Phi.z %*% solve(diag(q) - Phi.z) %*% mu.z,ncol=1),
                matrix(mu.z,ncol=1))
  Phi.X <- cbind(model$Phi.Y,model$Theta %*% Phi.z)
  Phi.X <- rbind(Phi.X,cbind(matrix(0,q,n),Phi.z))
  return(merge.list(model,list(r=r,n=n,q=q,mu.z=mu.z,Phi.z=Phi.z,Gamma.z0=Gamma.z0,Gamma.z1=Gamma.z1,
                               mu.X=mu.X,Phi.X=Phi.X)))
}

make.S.p <- function(p){
  Id <- diag(p)
  S.p <- matrix(0,p*p,p)
  for(i in 1:p){
    S.p <- S.p + (Id[,i]%x%Id[,i])%*%t(Id[,i])
  }
  return(S.p)
}

a.1 <- function(gamma,Model){
  return(sum(gamma*Model$mu.X))
}

b.1 <- function(gamma,Model){
  return(t(Model$Phi.X)%*%gamma)
}

alpha.1 <- function(gamma,Model,S.n,S.q){
  gamma.Y <- gamma[1:Model$n]
  gamma.z <- gamma[(Model$n+1):(Model$n+Model$q)]
  res <- matrix(gamma.Y %x% gamma.Y,nrow=1) %*% 
    ((Model$Theta %x% Model$Theta) %*% S.q %*% Model$Gamma.z0 +
       S.n %*% Model$Gamma.Y0 + S.n %*% t(Model$Gamma.Y1) %*% Model$mu.z) + 
    matrix(gamma.z %x% gamma.z,nrow=1) %*% 
    S.q %*% Model$Gamma.z0 +
    2*matrix(gamma.z %x% gamma.Y,nrow=1) %*% 
    (diag(Model$q) %x% Model$Theta) %*% S.q %*% Model$Gamma.z0
  return(res)
}

beta.1 <- function(gamma,Model,S.n,S.q){
  gamma.Y <- gamma[1:Model$n]
  gamma.z <- gamma[(Model$n+1):(Model$n+Model$q)]
  res <- t(matrix(gamma.Y %x% gamma.Y,nrow=1) %*% 
             ((Model$Theta %x% Model$Theta) %*% S.q %*% t(Model$Gamma.z1) +
                S.n %*% t(Model$Gamma.Y1) %*% Model$Phi.z) +
             matrix(gamma.z %x% gamma.z,nrow=1) %*% 
             S.q %*% t(Model$Gamma.z1) +
             2*matrix(gamma.z %x% gamma.Y,nrow=1) %*% 
             (diag(Model$q) %x% Model$Theta) %*% S.q %*% t(Model$Gamma.z1)
  )
  res <- rbind(matrix(0,Model$n,1),res)
  return(res)
}

compute.loadings <- function(Gamma,Model,H,S.n,S.q,indic.5y.in.Xy=0){
  # Compared to the previous function (compute.loadings), Gamma is not defined in the same way.
  # Here, loadings are computed for three cases:
  # 1. Inflation is average inflation between t and t+h, and inflation between t-12 and t is of the form Gamma'X_t
  # 2. Inflation between t+h-12 and t+h (simpler)
  # 3. "US.SPF-style": average of Price index for a given year over average Price index for the rpeceding year.
  #
  # indic.5y.in.Xy is equal to one if, for horizons higher
  #     than 5y, what is given (for case-1 inflation) are 5y-in-Xy inflation rates
  
  Gamma.2nd.case <- matrix(0,max(H),Model$n+Model$q)
  Gamma.2nd.case[max(H),] <- c(Gamma)
  Gamma.1st.case <- Gamma.2nd.case
  aux.index <- seq(max(H),1,by=-12)
  Gamma.1st.case[aux.index,] <- matrix(1,nrow=length(aux.index),1) %*% matrix(Gamma,nrow=1)
  
  Gamma.3rd.case <- matrix(0,max(H),Model$n+Model$q)
  aux.index <- seq(max(H),max(H)-11,by=-3)
  Gamma.3rd.case[aux.index,] <- 1/4*matrix(1,4,1) %*% matrix(Gamma,nrow=1)
  
  
  if(indic.5y.in.Xy==1){# Then, for horizons X>60mths, we consider 5y-in-Xmths rates
    if(dim(Gamma.1st.case)[1]>60){
      Gamma.1st.case[1:(dim(Gamma.1st.case)[1]-60),] <-
        0*Gamma.1st.case[1:(dim(Gamma.1st.case)[1]-60),]
    }
  }
  
  maxH <- max(H) # This should also correspond to the number of rows of Gamma
  nb.horizons <- length(c(H))
  a     <- array(NaN,c(1,nb.horizons,3)) # three layers for the three inflation types
  alpha <- array(NaN,c(1,nb.horizons,3)) # three layers for the three inflation types
  b     <- array(NaN,c(Model$n+Model$q,nb.horizons,3)) # three layers for the three inflation types
  beta  <- array(NaN,c(Model$n+Model$q,nb.horizons,3)) # three layers for the three inflation types
  
  count <- 0 # will count the number of requested maturities already treated
  for(h in 1:maxH){
    if(h==1){
      a.h.1st.case <- a.1(c(Gamma.1st.case[maxH,]),Model)
      b.h.1st.case <- b.1(c(Gamma.1st.case[maxH,]),Model)
      alpha.h.1st.case <- alpha.1(c(Gamma.1st.case[maxH,]),Model,S.n,S.q)
      beta.h.1st.case  <- beta.1(c(Gamma.1st.case[maxH,]),Model,S.n,S.q)
      
      a.h.2nd.case <- a.h.1st.case
      b.h.2nd.case <- b.h.1st.case
      alpha.h.2nd.case <- alpha.h.1st.case
      beta.h.2nd.case  <- beta.h.1st.case
      
      a.h.3rd.case <- a.1(c(Gamma.3rd.case[maxH,]),Model)
      b.h.3rd.case <- b.1(c(Gamma.3rd.case[maxH,]),Model)
      alpha.h.3rd.case <- alpha.1(c(Gamma.3rd.case[maxH,]),Model,S.n,S.q)
      beta.h.3rd.case  <- beta.1(c(Gamma.3rd.case[maxH,]),Model,S.n,S.q)
    }else{
      a.h.1st.case <- a.h.1st.case + a.1(c(Gamma.1st.case[maxH-h+1,]) + b.h.1st.case,Model)
      b.h.1st.case <- b.1(c(Gamma.1st.case[maxH-h+1,]) + b.h.1st.case,Model)
      alpha.h.1st.case <- alpha.1(c(Gamma.1st.case[maxH-h+1,]) + b.h.1st.case,Model,S.n,S.q) +
        alpha.h.1st.case + a.1(beta.h.1st.case,Model)
      beta.h.1st.case <- beta.1(c(Gamma.1st.case[maxH-h+1,]) + b.h.1st.case,Model,S.n,S.q) +
        b.1(beta.h.1st.case,Model)
      
      a.h.2nd.case <- a.h.2nd.case + a.1(c(Gamma.2nd.case[maxH-h+1,]) + b.h.2nd.case,Model)
      b.h.2nd.case <- b.1(c(Gamma.2nd.case[maxH-h+1,]) + b.h.2nd.case,Model)
      alpha.h.2nd.case <- alpha.1(c(Gamma.2nd.case[maxH-h+1,]) + b.h.2nd.case,Model,S.n,S.q) +
        alpha.h.2nd.case + a.1(beta.h.2nd.case,Model)
      beta.h.2nd.case <- beta.1(c(Gamma.2nd.case[maxH-h+1,]) + b.h.2nd.case,Model,S.n,S.q) +
        b.1(beta.h.2nd.case,Model)
      
      a.h.3rd.case <- a.h.3rd.case + a.1(c(Gamma.3rd.case[maxH-h+1,]) + b.h.3rd.case,Model)
      b.h.3rd.case <- b.1(c(Gamma.3rd.case[maxH-h+1,]) + b.h.3rd.case,Model)
      alpha.h.3rd.case <- alpha.1(c(Gamma.3rd.case[maxH-h+1,]) + b.h.3rd.case,Model,S.n,S.q) +
        alpha.h.3rd.case + a.1(beta.h.3rd.case,Model)
      beta.h.3rd.case <- beta.1(c(Gamma.3rd.case[maxH-h+1,]) + b.h.3rd.case,Model,S.n,S.q) +
        b.1(beta.h.3rd.case,Model)
    }
    
    if(sum(h==H)>0){
      # in that case, the loadings are expected in the output
      count <- count + 1
      deflator <- (1-indic.5y.in.Xy) * (1+as.integer((h-1)/12))+
        indic.5y.in.Xy * min(5,1+as.integer((h-1)/12))
      
      a[1,count,1] <- a.h.1st.case/deflator
      b[,count,1]  <- b.h.1st.case/deflator
      alpha[1,count,1] <- alpha.h.1st.case/deflator^2
      beta[,count,1]   <- beta.h.1st.case/deflator^2
      
      a[1,count,2] <- a.h.2nd.case
      b[,count,2]  <- b.h.2nd.case
      alpha[1,count,2] <- alpha.h.2nd.case
      beta[,count,2]   <- beta.h.2nd.case
      
      a[1,count,3] <- a.h.3rd.case
      b[,count,3]  <- b.h.3rd.case
      alpha[1,count,3] <- alpha.h.3rd.case
      beta[,count,3]   <- beta.h.3rd.case
    }
  }
  return(list(a = a, b = b, alpha = alpha, beta = beta))
}
# use compiler:
compute.loadings.c <- cmpfun(compute.loadings)



a_z <- function(Model,u_z){
  # u_z is of dimension Model$q x k
  # k is the number of u's at which the function is evaluated
  # The output is of dimension 1 x k
  if(class(u_z)=="matrix"){
    k <- dim(u_z)[2]
    mat.ones <- matrix(1,Model$q,k)
    mat.mu <- matrix(Model$mu,Model$q,k)
  }else{
    mat.ones <- 1
    mat.mu <- Model$mu
  }
  return(- Model$nu %*% log(mat.ones - mat.mu*u_z))
}

b_z <- function(Model,u_z){
  # u_z is of dimension Model$q x k
  # k is the number of u's at which the function is evaluated
  # The output is of dimension Model$q x k
  if(class(u_z)=="matrix"){
    k <- dim(u_z)[2]
    mat.ones <- matrix(1,Model$q,k)
    mat.mu <- matrix(Model$mu,Model$q,k)
  }else{
    mat.ones <- 1
    mat.mu <- Model$mu
  }
  return(Model$phi %*% (mat.mu*u_z/(mat.ones - mat.mu*u_z)))
}


A_1 <- function(Model,u_Y,u_z){
  # u_Y is of dimension Model$n x k
  # u_z is of dimension Model$q x k
  # k is the number of u's at which the function is evaluated
  z.bar <- solve(diag(Model$q)-Model$Phi.z)%*%Model$mu.z
  return(
    a_z(Model,u_z + t(Model$Theta)%*%u_Y + .5 * Model$Gamma.Y1 %*% u_Y^2) -
      t(t(u_Y) %*% Model$Theta %*% z.bar) + .5 * Model$Gamma.Y0 %*% u_Y^2
  )
}

B_1 <- function(Model,u_Y,u_z){
  # u_Y is of dimension Model$n x k
  # u_z is of dimension Model$q x k
  # k is the number of u's at which the function is evaluated
  return(rbind(
    t(Model$Phi.Y) %*% u_Y,
    b_z(Model,u_z + t(Model$Theta)%*%u_Y + .5 * Model$Gamma.Y1 %*% u_Y^2)
  ))
}

compute_AB <- function(Model,u_Y,u_z,HH,
                       indic.average=0){
  # U_Y is of dimension Model$n x k
  # U_z is of dimension Model$q x k
  # k is the number of u's at which the function is evaluated
  # H is a vector indicating the horizons
  # If indic.average = 1, one considers E(exp(u'[X{+HH}+X{+HH-12}+X{+HH-24}+...]))
  k <- dim(u_Y)[2]
  nb.H <- length(HH)
  A <- array(NaN,c(1,k,nb.H))
  B <- array(NaN,c(Model$n+Model$q,k,nb.H))
  count <- 0
  for(h in 1:max(HH)){
    if(indic.average==2){# In that case, third case of inflation (US SPF type)
      if(h==1){
        A.h <- A_1(Model,u_Y,u_z)
        B.h <- B_1(Model,u_Y,u_z)
      }else{
        if(((h-1)/3 == as.integer((h-1)/3))&(h<12)){
          u.1.Y <- u_Y
          u.1.z <- u_z
        }else{
          u.1.Y <- 0*u_Y
          u.1.z <- 0*u_z
        }
        A.h <- A.h_1 + A_1(Model,
                           u.1.Y + B.h_1[1:(Model$n),],
                           u.1.z + B.h_1[(Model$n+1):(Model$n+Model$q),])
        B.h <- B_1(Model,
                   u.1.Y + B.h_1[1:(Model$n),],
                   u.1.z + B.h_1[(Model$n+1):(Model$n+Model$q),])
      }
    }else{
      if(h==1){
        A.h <- A_1(Model,u_Y,u_z)
        B.h <- B_1(Model,u_Y,u_z)
      }else{
        if((indic.average==1)&((h-1)/12 == as.integer((h-1)/12))){
          if((Indic.5y.in.Xy==1)&(h>61)){
            u.1.Y <- 0*u_Y
            u.1.z <- 0*u_z
          }else{
            u.1.Y <- u_Y
            u.1.z <- u_z
          }
        }else{
          u.1.Y <- 0*u_Y
          u.1.z <- 0*u_z
        }
        A.h <- A.h_1 + A_1(Model,
                           u.1.Y + B.h_1[1:(Model$n),],
                           u.1.z + B.h_1[(Model$n+1):(Model$n+Model$q),])
        B.h <- B_1(Model,
                   u.1.Y + B.h_1[1:(Model$n),],
                   u.1.z + B.h_1[(Model$n+1):(Model$n+Model$q),])
      }
    }
    A.h_1 <- A.h
    B.h_1 <- B.h
    if(sum(h==HH)>0){
      count <- count + 1
      A[,,count] <- c(A.h)
      B[,,count] <- c(B.h)
    }
  }
  return(list(A=A,B=B))
}

compute.phi.pi <- function(Model,u_pi,HH,indic.average=0){
  # This function computes the multi-horizon Laplace transform of the different inflation rates.
  # u_pi is of dimension (r x k)
  # k is the number of points where we want to compute the Laplance transform.
  r <- Model$r
  k <- dim(u_pi)[2]
  
  u_Y <- Model$delta %*% u_pi
  u_z <- matrix(0,Model$q,k)
  
  AB.list <- compute_AB(Model,u_Y,u_z,HH,indic.average)
  
  return(list(A=AB.list$A,B=AB.list$B))
}

compute.matrices.4.condi.covar <- function(Model,HH,indic.average=0){
  # HH must be scalar 
  r <- Model$r
  eps <- .0000001
  all.A <- matrix(NaN,r^2,1)
  all.B <- matrix(NaN,r^2,n+q)
  count <- 0
  for(i in 1:r){
    for(j in 1:r){
      count <- count + 1
      u_pi <- matrix(0,r,3)
      u_pi[i,1] <- eps
      u_pi[j,2] <- eps
      u_pi[i,3] <- eps
      u_pi[j,3] <- u_pi[j,3] + eps
      AB.list <- compute.phi.pi(Model,u_pi,HH,indic.average)
      all.A[count,] <- (AB.list$A[1,3,1] + 0 - AB.list$A[1,2,1] - AB.list$A[1,1,1])/(eps^2)
      all.B[count,] <- (AB.list$B[ ,3,1] + 0 - AB.list$B[ ,2,1] - AB.list$B[ ,1,1])/(eps^2)
    }
  }
  return(list(all.A=all.A,all.B=all.B))
}


compute.Var.f.X <- function(f,X,S,OPTIONS){
  # f is a function of X and we want to compute Var(f(X))
  # X is T x (n + q) and S contains the variances matrices of X, it is therefore of dimension (T x (n+q)^2)
  k <- dim(X)[2]
  T <- dim(X)[1]
  eps <- .000001
  f.X <- f(X,OPTIONS)
  grad.f <- matrix(NaN,T,k)
  for(i in 1:k){
    print(paste("compute grad wrt component",toString(i)," of X",sep=""))
    X.aux <- X
    X.aux[,i] <- X.aux[,i] + eps
    grad.f[,i] <- (f(X.aux,OPTIONS) - f.X)/eps
  }
  aux1 <- grad.f %x% matrix(1,1,k)
  aux2 <- matrix(1,1,k) %x% grad.f
  Var.f.X <- apply(S * aux1 * aux2,1,sum)
  return(Var.f.X)
}

f.compute.var.EA <- function(X,OPTIONS){
  Model <- OPTIONS$Model
  HH <- OPTIONS$HH
  res <- compute.matrices.4.condi.covar(Model,HH)
  T <- dim(X)[1]
  computed.var <- matrix(1,T,1) %*% matrix(res$all.A,nrow=1) +
    X %*% t(res$all.B)
  return(computed.var[,1])
}

f.compute.var.US <- function(X,OPTIONS){
  Model <- OPTIONS$Model
  HH <- OPTIONS$HH
  res <- compute.matrices.4.condi.covar(Model,HH)
  T <- dim(X)[1]
  computed.var <- matrix(1,T,1) %*% matrix(res$all.A,nrow=1) +
    X %*% t(res$all.B)
  return(computed.var[,4])
}

f.compute.stdv.EA <- function(X,OPTIONS){
  Model <- OPTIONS$Model
  HH <- OPTIONS$HH
  res <- compute.matrices.4.condi.covar(Model,HH)
  T <- dim(X)[1]
  computed.var <- matrix(1,T,1) %*% matrix(res$all.A,nrow=1) +
    X %*% t(res$all.B)
  return(sqrt(computed.var[,1]))
}

f.compute.stdv.US <- function(X,OPTIONS){
  Model <- OPTIONS$Model
  HH <- OPTIONS$HH
  res <- compute.matrices.4.condi.covar(Model,HH)
  T <- dim(X)[1]
  computed.var <- matrix(1,T,1) %*% matrix(res$all.A,nrow=1) +
    X %*% t(res$all.B)
  return(sqrt(computed.var[,4]))
}

f.compute.covar <- function(X,OPTIONS){
  Model <- OPTIONS$Model
  HH <- OPTIONS$HH
  res <- compute.matrices.4.condi.covar(Model,HH)
  T <- dim(X)[1]
  computed.var <- matrix(1,T,1) %*% matrix(res$all.A,nrow=1) +
    X %*% t(res$all.B)
  return(computed.var[,2])
}

f.compute.correl <- function(X,OPTIONS){
  Model <- OPTIONS$Model
  HH <- OPTIONS$HH
  res <- compute.matrices.4.condi.covar(Model,HH)
  T <- dim(X)[1]
  computed.var <- matrix(1,T,1) %*% matrix(res$all.A,nrow=1) +
    X %*% t(res$all.B)
  return(computed.var[,2] / sqrt(computed.var[,1]*computed.var[,4]))
}


compute.proba <- function(Model,X,b,y,HH,step,max.v,
                          indic.average=0,
                          AB.list.external=NaN,
                          Indic.5y.in.Xy=1){ #used in the function below compute.distri.plus.stdv
  # Computes the proba that bX is below y at horizons given in HH, ie,
  # E(1{b' * X_{+HH} < y}|X)
  # based on Duffie, Pan and Singleton (2000, Econometrica), formula 2.12
  #
  # step and max.step are used to numerically evaluate the integral of the formula:
  # i.e. v = .000001, step, 2 step, ..., max.v
  #
  # X is a matrix of dimension T x (Model$n + Model$q)
  # 
  # If indic.average = 1, then one will consider:
  #    E(1{b'(X_{+HH} + X_{+HH-12} + X_{+HH-24} + ...) < (HH/12) * y}|X)
  # 
  Proba <- NULL
  T <- dim(X)[1]
  max.x <- log(max.v)
  v <- matrix(exp(seq(-10,max.x,by=step)),nrow=1)
  dim.v <- length(v)
  if(!is.list(AB.list.external)){# recompute Laplace transform
    u_Y <- 1i * matrix(b[1:Model$n],ncol=1) %*% v  # our paper, section 7.3 Appendix i*v*gamma for Y
    u_z <- 1i * matrix(b[(Model$n+1):(Model$n+Model$q)],ncol=1) %*% v  # our paper, section 7.3 Appendix i*v*gamma for z
    AB.list <- compute_AB(Model,u_Y,u_z,HH,indic.average)
  }else{# use Laplace transform already computed
    AB.list <- AB.list.external
  }
  if(indic.average==1){
    Y <- y * as.integer((HH-Indic.5y.in.Xy*(HH-60)*(HH>60))/12) # because in that case (indic.average==1), compute_AB
  }else if(indic.average==2){# in that case (indic.average==2), compute_AB
    Y <- rep(y*4,length(HH))
  }else{
    Y <- rep(y,length(HH))
  }
  psi.B <- apply(AB.list$B,3,function(layer){X %*% layer})
  # at that stage psi is a matrix of dimension (T*dim(v)) x nb.HH
  count <- 0
  extended.v <- cbind(0,v,max.v)
  dim.ext.v <- dim.v + 2
  weights <- matrix(.5*(extended.v[2:(dim.ext.v-1)]-extended.v[1:(dim.ext.v-2)])+
                      .5*(extended.v[3:dim.ext.v]-extended.v[2:(dim.ext.v-1)]),nrow=1)
  for(h in HH){
    count <- count + 1
    psi <- exp(
      matrix(1,T,1) %*% AB.list$A[,,count] + matrix(psi.B[,count],T,dim.v)
    )
    numerator <- Im(psi * (matrix(1,T,1) %*% exp(-1i*v*Y[count])))
    integrand <- numerator / (matrix(1,T,1) %*% v)
    Proba <- cbind(Proba, .5 - 1/pi * integrand %*% t(weights))
  }
  return(Proba)
}


compute.distri.plus.stdv <- function(Model,X,b,HH,  # used in compute.analytical.distri
                                     min.bx,max.bx,step.distri,
                                     step.4.integral,
                                     max.v.4.integral,
                                     indic.average=0,
                                     Sigma.X=0,
                                     AB.list.external=NaN){
  # Computes the distribution of b'X
  # HH should be of dimension 1.
  # X is of dimension T x (Model$n + Model$q)
  # Sigma.X is of dimension T x ((Model$n + Model$q) x (Model$n + Model$q))
  # If Sigma.X = 0, no standard deviation for the distri function is computed.
  # min.bx (mas.bx) is the minimun (maximum) considered value of the random variable b'X
  # The distrinbution function is evaluated J points, where J = (max.bx - min.bx)/step.distri
  # The output is a list that contains:
  #     - the matrix (PDF) of evaluations of the distribution function (dimension T x J)
  #     - the matrix (PDF.stdv) of std. dev, of the evaluations of the distribution function (dimension T x J)
  #       (this stdv takes into account the uncertainty of the Xs, as relfected in Sigma.X)
  #     - the vector (PDF.x) of points where the distribution is evaluated
  
  vec.y <- seq(min.bx,max.bx,by=step.distri)
  CDF <- NULL
  for(iii in 1:length(vec.y)){
    y <- vec.y[iii]
    #print(y)
    aux <- compute.proba(Model,X,b,y,HH,
                         step.4.integral,
                         max.v.4.integral,
                         indic.average,
                         AB.list.external,
                         Indic.5y.in.Xy)
    CDF <- cbind(CDF,aux)
  }
  PDF <- (CDF[,3:dim(CDF)[2]] - CDF[,1:(dim(CDF)[2]-2)])/2  # [F(x2)-F(x1)]/[x2-x1]
  PDF.x <- vec.y[2:(length(vec.y)-1)]
  if(length(Sigma.X)==1){
    Grad.PDF <- NULL
    Grad.CDF <- NULL
    PDF.stdv <- NULL
    CDF.stdv <- NULL
  }else{
    # Computation of the stdv of the proba by the Delta method.
    
    # ================================
    # Step 1: Compute the gradient of the distri matrix wrt each of the X ->
    # The gradient is put in a 3d array of dimension T x J x (n + q)
    Grad.PDF <- array(NaN,c(dim(X)[1],length(PDF.x),Model$n+Model$q))
    Grad.CDF <- array(NaN,c(dim(X)[1],length(PDF.x)+2,Model$n+Model$q))
    
    # computation of the marginal standard deviations of the Xs
    # (will be used to calibrate the perturbation when computing the gradient of the proba wrt the Xs)
    stdv.X <- sqrt(diag(unc.moments.X(Model)$X.mom.2))
    
    for(dim.X in 1:(Model$n + Model$q)){
      epsilon <- stdv.X[dim.X]/100
      CDF.aux <- NULL
      for(iii in 1:length(vec.y)){
        y <- vec.y[iii]
        print(c(dim.X,y))
        X.aux <- X
        X.aux[,dim.X] <- X.aux[,dim.X] + epsilon
        aux <- compute.proba(Model,X.aux,b,y,
                             HH,step.4.integral,
                             max.v.4.integral,
                             indic.average,
                             AB.list.external,
                             Indic.5y.in.Xy)
        CDF.aux <- cbind(CDF.aux,aux)
      }
      Grad.PDF[,,dim.X] <- (
        (CDF.aux[,3:dim(CDF)[2]] - CDF.aux[,1:(dim(CDF)[2]-2)])/2
        - PDF
      )/epsilon
      Grad.CDF[,,dim.X] <- (CDF.aux - CDF)/epsilon
    }
    PDF.stdv <- NULL
    CDF.stdv <- NULL
    for(x in 1:dim(Grad.PDF)[2]){
      Grad.PDF.aux <- Grad.PDF[,x,]
      Grad.Grad.PDF <- t(apply(Grad.PDF.aux,1,function(x){x %x% x})) # each row is the outer-product of the rows of Grad
      aux <- apply(Grad.Grad.PDF * Sigma.X,1,sum)  # delta method: var[f(x)]=[f'(x)]^2 * sigma_X^2
      PDF.stdv <- cbind(PDF.stdv,sqrt(aux))
      Grad.CDF.aux <- Grad.CDF[,x+1,]
      Grad.Grad.CDF <- t(apply(Grad.CDF.aux,1,function(x){x %x% x})) # each row is the outer-product of the rows of Grad
      aux <- apply(Grad.Grad.CDF * Sigma.X,1,sum)
      CDF.stdv <- cbind(CDF.stdv,sqrt(aux))
    }
  }
  return(list(CDF=CDF[,2:(dim(CDF)[2]-1)],PDF=PDF,PDF.x=PDF.x,
              PDF.stdv=PDF.stdv,
              CDF.stdv=CDF.stdv,
              Grad.PDF=Grad.PDF,
              Grad.CDF=Grad.CDF))
}

compute.proba.interval.plus.stdv <- function(Model,X,b,HH,  # used in compute.analytical.distri
                                             lower.bound,upper.bound,
                                             step.4.integral,max.v.4.integral,indic.average=0,
                                             Sigma.X=0,AB.list.external=NaN){
  # Computes the distribution of b'X
  # HH should be of dimension 1.
  # X is of dimension T x (Model$n + Model$q)
  # Sigma.X is of dimension T x ((Model$n + Model$q) x (Model$n + Model$q))
  # If Sigma.X = 0, no standard deviation for the distri function is computed.
  # min.bx (mas.bx) is the minimun (maximum) considered value of the random variable b'X
  # The distrinbution function is evaluated J points, where J = (max.bx - min.bx)/step.distri
  # The output is a list that contains:
  #     - the matrix (PDF) of evaluations of the distribution function (dimension T x J)
  #     - the matrix (PDF.stdv) of std. dev, of the evaluations of the distribution function (dimension T x J)
  #       (this stdv takes into account the uncertainty of the Xs, as relfected in Sigma.X)
  #     - the vector (PDF.x) of points where the distribution is evaluated
  
  cdf.lower <- compute.proba(Model,X,b,lower.bound,HH,step.4.integral,
                             max.v.4.integral,indic.average,AB.list.external)
  cdf.upper <- compute.proba(Model,X,b,upper.bound,HH,step.4.integral,
                             max.v.4.integral,indic.average,AB.list.external)
  Proba <- cdf.upper - cdf.lower
  if(length(Sigma.X)==1){
    Proba.stdv <- NaN
  }else{
    # Computation of the stdv of the proba by the Delta method.
    
    # computation of the marginal standard deviations of the Xs
    # (will be used to calibrate the perturbation when computing the gradient of the proba wrt the Xs)
    stdv.X <- sqrt(diag(unc.moments.X(Model)$X.mom.2))
    
    Grad.Proba <- matrix(NaN,length(Proba),Model$n + Model$q)
    for(dim.X in 1:(Model$n + Model$q)){
      epsilon <- stdv.X[dim.X]/100
      
      X.aux <- X
      X.aux[,dim.X] <- X.aux[,dim.X] + epsilon
      
      cdf.lower <- compute.proba(Model,X.aux,b,lower.bound,HH,step.4.integral,
                                 max.v.4.integral,indic.average,AB.list.external)
      cdf.upper <- compute.proba(Model,X.aux,b,upper.bound,HH,step.4.integral,
                                 max.v.4.integral,indic.average,AB.list.external)
      Proba.aux <- cdf.upper - cdf.lower
      
      Grad.Proba[,dim.X] <- (Proba.aux - Proba)/epsilon
    }
    Grad.Grad.Proba <- t(apply(Grad.Proba,1,function(x){x %x% x}))
    Proba.stdv <- sqrt(apply(Grad.Grad.Proba * Sigma.X,1,sum))
    
  }
  return(list(Proba=Proba,
              Proba.stdv=Proba.stdv))
}


compute.difference.proba.plus.stdv <- function(Model,X,HH,  # used in compute.analytical.distri
                                               lower.bound,upper.bound,
                                               step.4.integral,max.v.4.integral,indic.average=0,
                                               Sigma.X=0,AB.list.external.1,AB.list.external.2,
                                               wedge=0){
  # HERE, bounds are expressed with "true" exprssions, i.e. not relative to average pi.
  # wedge accounts for a potential shift in the intervals associated with the different countries
  
  for(area in 1:2){
    
    if(area==1){
      AB.list.external <- AB.list.external.1
    }else{
      AB.list.external <- AB.list.external.2
    }
    
    b <- c(Model$delta[,area],rep(0,Model$q))
    lower.bound.bX <- lower.bound - Model$pi.bar[area] + ifelse(area==2,wedge,0)
    upper.bound.bX <- upper.bound - Model$pi.bar[area] + ifelse(area==2,wedge,0)
    
    cdf.lower <- compute.proba(Model,X,b,lower.bound.bX,HH,step.4.integral,
                               max.v.4.integral,indic.average,AB.list.external)
    cdf.upper <- compute.proba(Model,X,b,upper.bound.bX,HH,step.4.integral,
                               max.v.4.integral,indic.average,AB.list.external)
    if(area==1){
      Proba.10 <- cdf.upper - cdf.lower
    }else{
      Proba.20 <- cdf.upper - cdf.lower
    }
  }
  Proba.diff <- Proba.10 - Proba.20
  if(length(Sigma.X)==1){
    Proba.diff.stdv <- NaN
  }else{
    # Computation of the stdv of the proba by the Delta method.
    
    # computation of the marginal standard deviations of the Xs
    # (will be used to calibrate the perturbation when computing the gradient of the proba wrt the Xs)
    stdv.X <- sqrt(diag(unc.moments.X(Model)$X.mom.2))
    
    Grad.Proba <- matrix(NaN,length(Proba.diff),Model$n + Model$q)
    for(dim.X in 1:(Model$n + Model$q)){
      epsilon <- stdv.X[dim.X]/100
      
      X.aux <- X
      X.aux[,dim.X] <- X.aux[,dim.X] + epsilon
      
      for(area in 1:2){
        
        if(area==1){
          AB.list.external <- AB.list.external.1
        }else{
          AB.list.external <- AB.list.external.2
        }
        
        b <- c(Model$delta[,area],rep(0,Model$q))
        lower.bound.bX <- lower.bound - Model$pi.bar[area] + ifelse(area==2,wedge,0)
        upper.bound.bX <- upper.bound - Model$pi.bar[area] + ifelse(area==2,wedge,0)
        
        cdf.lower <- compute.proba(Model,X.aux,b,lower.bound.bX,HH,step.4.integral,
                                   max.v.4.integral,indic.average,AB.list.external)
        cdf.upper <- compute.proba(Model,X.aux,b,upper.bound.bX,HH,step.4.integral,
                                   max.v.4.integral,indic.average,AB.list.external)
        if(area==1){
          Proba.1 <- cdf.upper - cdf.lower
        }else{
          Proba.2 <- cdf.upper - cdf.lower
        }
      }
      Proba.diff.aux <- Proba.1 - Proba.2
      
      Grad.Proba[,dim.X] <- (Proba.diff.aux - Proba.diff)/epsilon
    }
    Grad.Grad.Proba <- t(apply(Grad.Proba,1,function(x){x %x% x}))
    Proba.diff.stdv <- sqrt(apply(Grad.Grad.Proba * Sigma.X,1,sum))
    
  }
  return(list(Proba.diff=Proba.diff,
              Proba.diff.stdv=Proba.diff.stdv,Proba.1=Proba.10,Proba.2=Proba.20))
}


compute.two.moments <- function(Model,X,HH,S.provided=0, indic.5y.in.Xy = 0){
  # This function returns the series of the first two moments associated to
  # pi.bar + delta'X_{t+h} conditional to the information available at t.
  # Note: HH is scalar
  # If X is of dimension k=(n+q) S is of dimension T x (k^2), the t^th row corresponds 
  # to the covariance matrix of X_t (smmothing error).
  T <- dim(X)[1] # Length of the state factors
  k <- dim(X)[2]
  if(length(S.provided)==1){# In that case, no covariance matrix of the X_t's has been provided
    S <- matrix(0,T,k^2)
  }else{
    S <- S.provided
  }
  Gamma <- rbind(Model$delta,matrix(0,Model$q,Model$r))
  S.n <- make.S.p(Model$n)
  S.q <- make.S.p(Model$q)
  expect.1st.mom <- array(NaN,c(T,Model$r,3)) # one layer per inflation concept.
  expect.2nd.mom <- array(NaN,c(T,Model$r,3)) # one layer per inflation concept.
  var.1st.mom <- array(NaN,c(T,Model$r,3)) # one layer per inflation concept.
  var.2nd.mom <- array(NaN,c(T,Model$r,3)) # one layer per inflation concept.
  for(area in 1:Model$r){
    loadings <- compute.loadings.c(c(Gamma[,area]),Model,HH,S.n,S.q,indic.5y.in.Xy)
    for(infl.type in 1:3){
      expect.1st.mom[,area,infl.type] <- Model$pi.bar[area] + 
        loadings$a[1,1,infl.type] + X %*% loadings$b[,1,infl.type]
      alpha <- loadings$b[,1,infl.type]
      var.1st.mom[,area,infl.type] <- S %*% (alpha %x% alpha)
      expect.2nd.mom[,area,infl.type] <- 
        loadings$alpha[1,1,infl.type] + X %*% loadings$beta[,1,infl.type]
      alpha <- loadings$beta[,1,infl.type]
      var.2nd.mom[,area,infl.type] <- S %*% (alpha %x% alpha)
    }
  }
  return(list(expect.1st.mom=expect.1st.mom,expect.2nd.mom=expect.2nd.mom,
              var.1st.mom=var.1st.mom,var.2nd.mom=var.2nd.mom))
}



rVARG <- function(nb.sim,Model,z0){
  # z0 is the initial value of the simulation
  # Notations: see our Appendix
  q <- dim(z0)[2] # dimension of the vector to simulate
  T <- dim(z0)[1]
  Ones <- matrix(1,q,1)
  z    <- array(0,c(T,q,nb.sim+1))
  z[,,1] <- z0
  for (i in 2:(nb.sim+1)){
    z_1   <- z[,,i-1]
    for(j in 1:q){
      Z_poisson <- rpois(T,as.matrix(z_1) %*% Model$phi[,j])
      z[,j,i] <- rgamma(T,shape = Model$nu[j] + Z_poisson,scale = Model$mu[j])
    }
  }
  return(z)
}


unc.moments <- function(Model,S.n.input=NaN,S.q.input=NaN){
  if(is.na(S.n.input[1])){
    S.n <- make.S.p(Model$n)
    S.q <- make.S.p(Model$q)
  }else{
    S.n <- S.n.input
    S.q <- S.q.input
  }
  z.mom.1 <- solve(diag(Model$q)-Model$Phi.z)%*%Model$mu.z
  z.vec.mom.2 <- solve(diag((Model$q)^2)-Model$Phi.z%x%Model$Phi.z)%*%S.q %*%(Model$Gamma.z0+t(Model$Gamma.z1)%*%z.mom.1)
  z.mom.2 <- matrix(z.vec.mom.2, Model$q, Model$q)
  return(list(z.mom.1 = z.mom.1, z.mom.2 = z.mom.2))
}

unc.moments.X <- function(Model,S.n.input=NaN,S.q.input=NaN){
  if(is.na(S.n.input[1])){
    S.n <- make.S.p(Model$n)
    S.q <- make.S.p(Model$q)
  }else{
    S.n <- S.n.input
    S.q <- S.q.input
  }
  unc.mom.z <- unc.moments(Model,S.n,S.q) # Computes the unconditional moments of z
  X.0   <- c(rep(0,Model$n),unc.mom.z$z.mom.1) # unconditional mean of the state vector
  aux.Sigma0 <- Qfunction(Model,X.0) # Computes the unconditional value of the conditional corvariance matrix
  Sigma.0.vec <- ginv(diag((Model$n+Model$q)^2) - Model$Phi.X%x%Model$Phi.X) %*% c(aux.Sigma0)
  Sigma.0 <- matrix(Sigma.0.vec,Model$n+Model$q,Model$n+Model$q)
  return(list(X.mom.1 = X.0, X.mom.2 = Sigma.0))
}




# transform vector into parameter matrices
theta.to.Model <- function(theta,r,n,q,k=0,indices.of.addit.var=NULL){
  # k is the number of additional measurement equations 
  full.theta <- eval.parent(full.theta)
  full.theta.local <- full.theta
  full.theta.local[Filter>=1] <- theta
  
  count0 <- r
  pi.bar <- full.theta.local[1:count0]
  
  count1 <- count0+(n*r)
  delta <- matrix(full.theta.local[(count0+1):count1],n,r)
  
  count2 <- count1+(n*n)
  Phi.Y <- matrix(full.theta.local[(count1+1):count2],n,n)
  diag(Phi.Y) <- exp(diag(Phi.Y))/(1+exp(diag(Phi.Y)))

  count3 <- count2+(n*q)
  Theta <- matrix(full.theta.local[(count2+1):count3],n,q)/1000
  
  count4 <- count3+(n)
  Gamma.Y0 <- abs(full.theta.local[(count3+1):count4])/1000
  
  count5 <- count4+(q*n)
  Gamma.Y1 <- matrix(abs(full.theta.local[(count4+1):count5]),q,n)/1000
  
  count6 <- count5+q
  nu <- abs(full.theta.local[(count5+1):count6])
  
  count7 <- count6+q^2
  phi <- matrix(abs(full.theta.local[(count6+1):count7]),q,q)
  diag(phi) <- exp(diag(phi))/(1+exp(diag(phi)))
  
  mu <- rep(1,q)
  
  model <- list(pi.bar=pi.bar, delta=delta, Phi.Y=Phi.Y, Theta=Theta, Gamma.Y0=Gamma.Y0, Gamma.Y1=Gamma.Y1, nu=nu, phi=phi, mu=mu)
  
  Model <- make.Model(model)

  # ========================================
  # Additional terms for state-space model:
  # ========================================
  # Measurement errors (inflation-level equations):
  count8 <- count7 + sum(select.inflation.types>=1,na.rm=TRUE)
  aux.sigma.av <- abs(full.theta.local[(count7+1):count8])
  sigma.av <- NaN * select.inflation.types
  sigma.av[!is.na(select.inflation.types)&select.inflation.types>=1] <- aux.sigma.av
  # Measurement errors (inflation-variance equations):
  count9 <- count8 + sum(select.inflation.types==11,na.rm=TRUE)
  aux.sigma.var <- abs(full.theta.local[(count8+1):count9])
  sigma.var <- NaN * select.inflation.types
  sigma.var[!is.na(select.inflation.types)&select.inflation.types==11] <- aux.sigma.var
  if(k>0){
    # Measurement errors (additional measurement equations):
    count10 <- count9 + k
    sigma.addit <- abs(full.theta.local[(count9+1):count10])
    # Parameters for additional measurement equations:
    count11 <- count10 + k
    w <- full.theta.local[(count10+1):count11]
    count12 <- count11 + k*(n+q)
    W <- matrix(full.theta.local[(count11+1):count12],k,n+q)
  }else{
    sigma.addit <- NULL
    w <- NULL
    W <- NULL
  }
  Model <- merge.list(Model,list(sigma.av=sigma.av, sigma.var=sigma.var,
                                 sigma.addit=sigma.addit,w=w,W=W,
                                 indices.of.addit.var=indices.of.addit.var))
  return(Model) #fix
}


# Assign parameter matrices into the vector of parameters
Model.to.theta <- function(Model){
  # total number of parameters in full theta
  theta<- NULL
  theta <- c(theta,Model$pi.bar)
  theta <- c(theta,Model$delta)
  Phi.Y.aux <- Model$Phi.Y
  indic.is.0 <- which(diag(Phi.Y.aux)==0)
  indic.is.1 <- which(diag(Phi.Y.aux)==1)
  aux <- log(diag(Phi.Y.aux)/(1-diag(Phi.Y.aux)))
  aux[indic.is.0] <- 0
  aux[indic.is.1] <- 1000
  diag(Phi.Y.aux) <- aux
  theta <- c(theta,Phi.Y.aux)
  theta <- c(theta,Model$Theta*1000)
  theta <- c(theta,Model$Gamma.Y0*1000)  
  theta <- c(theta,Model$Gamma.Y1*1000)
  theta <- c(theta,Model$nu)
  phi.aux <- Model$phi
  indic.is.0 <- which(diag(phi.aux)==0)
  indic.is.1 <- which(diag(phi.aux)==1)
  aux <- log(diag(phi.aux)/(1-diag(phi.aux)))
  aux[indic.is.0] <- 0
  aux[indic.is.1] <- 1000
  diag(phi.aux) <- aux
  theta <- c(theta,phi.aux)
  theta <- c(theta,Model$sigma.av[!is.na(Model$sigma.av)])
  theta <- c(theta,Model$sigma.var[!is.na(Model$sigma.var)])
  
  theta <- c(theta,Model$sigma.addit)
  theta <- c(theta,Model$w)
  theta <- c(theta,Model$W)
  
  return(theta) 
}


interporlate.quantile <- function(CDF,q.values,x.values){
  spl <- smooth.spline(CDF,x.values,nknots=30)
  aux <- predict(spl,q.values)$y
  return(aux)
}

interpolate.proba <- function(CDF,threshold.values,x.values){
  spl <- smooth.spline(x.values,CDF,nknots=30)
  aux <- predict(spl,threshold.values)$y
  return(aux)
}

incomplete.beta <- function(x,a,b){
  return(pbeta(x,a,b)*beta(a,b))
}


cdf.incompl.beta <- function(x,param){
  a <- 1+abs(param[1])
  b <- 1+abs(param[2])
  c <- param[3]
  d <- c+abs(param[4])
  return(
    incomplete.beta((x-c)/(d-c),a,b)/beta(a,b)
  )
}

cdf.incompl.beta.vectorial <- function(x,param){
  a <- 1+abs(param[,1])
  b <- 1+abs(param[,2])
  c <- param[,3]
  d <- c+abs(param[,4])
  return(
    incomplete.beta((x-c)/(d-c),a,b)/beta(a,b)
  )
}

